package MyBlog::Controller::EntryCollection;

use strict;
use warnings;
use base qw(Catalyst::Controller::Atompub::Collection);

sub get_feed :Atompub(list) {
    my($self, $c) = @_;

    # フィード (XML::Atom::Feed) のひな型
    my $feed = $self->collection_resource->body;

    # entries テーブルからエントリを取得する (更新日時の新しい順)
    my $rs = $c->model('DBIC::Entries')
               ->search({}, { order_by => 'edited DESC' });

    # フィードのひな形にエントリを追加する
    while (my $entry_resource = $rs->next) {
        my $entry = XML::Atom::Entry->new(\$entry_resource->xml);
        $feed->add_entry($entry);
    }

    # 成功したら true を返す
    return 1;
}

sub create_resource :Atompub(create) {
    my($self, $c) = @_;

    # スーパークラスが自動生成した URI と更新日時を取得する
    my $uri    = $self->entry_resource->uri;
    my $edited = $self->edited->epoch; # (UNIX time 形式に変換)

    # POST されたエントリ本体を取得する (XML::Atom::Entry)
    my $entry = $self->entry_resource->body;

    # entries テーブルにエントリとメタデータを格納する
    $c->model('DBIC::Entries')->update_or_create({
        edited => $edited,
        uri    => $uri,
        xml    => $entry->as_xml,
    });

    # 成功したら true を返す
    return 1;
}

sub get_resource :Atompub(read) {
    my( $self, $c) = @_;

    # リクエストされた URI
    my $uri = $c->req->uri;

    # entries テーブルからエントリを検索する
    my $rs = $c->model('DBIC::Entries')->find({ uri => $uri });

    # エントリリソースをセットする
    my $entry = XML::Atom::Entry->new(\$rs->xml);
    $self->entry_resource->body($entry);

    # 成功したら true を返す
    return 1;
}

sub update_entry :Atompub(update) {
    my($self, $c) = @_;

    # リクエストされた URI
    my $uri = $c->req->uri;

    # スーパークラスが生成した更新日時を取得する
    my $edited = $self->edited->epoch; # (UNIX time 形式に変換)

    # PUT されたエントリ本体を取得する (XML::Atom::Entry)
    my $entry = $self->entry_resource->body;

    # entries テーブルからエントリを検索し，更新する
    $c->model('DBIC::Entries')->search({ uri => $uri })->update({
        edited => $edited,
        xml    => $entry->as_xml,
    });

    # 成功したら true を返す
    return 1;
}

sub delete_entry :Atompub(delete) {
    my($self, $c) = @_;

    # リクエストされた URI
    my $uri = $c->req->uri;

    # entries テーブルからエントリを削除する
    $c->model('DBIC::Entries')->search({ uri => $uri })->delete;

    # 成功したら true を返す
    return 1;
}

# Make EditURI by yourself (it will be automatically generated by default)
#sub make_edit_uri {
#    my($self, $c, @args) = @_;
#}

# Find version information of the Resource for caching and versioning
#sub find_version {
#    my($self, $c, $uri) = @_;
#}

=head1 NAME

MyBlog::Controller::EntryCollection - Atom Collection Resources

=head1 SYNOPSIS

See L<MyBlog>.

=head1 DESCRIPTION

Atom Collection Resources class.

=head1 AUTHOR

Takeru INOUE

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
